@@ -101,7 +101,7 @@ more target objects.
  *  AtkObject in a sequential way, (for instance text-flow).
  *@SPI_RELATION_FLOWS_FROM: Indicates that the object has content that flows logically from
  *  another AtkObject in a sequential way, (for instance text-flow).
- *@SPI_RELATION_SUBWINDOW_OF: 
+ *@SPI_RELATION_SUBWINDOW_OF: Indicates a subwindow attached to a component but otherwise has no connection in the UI heirarchy to that component.
  *@SPI_RELATION_EMBEDS: Indicates that the object visually embeds
  *  another object's content, i.e. this object's content flows around
  *  another's content.
@@ -221,6 +221,13 @@ typedef enum {
 typedef unsigned long AccessibleKeyEventMask;
 typedef unsigned long AccessibleDeviceEventMask;
 
+typedef struct {
+	long x;
+	long y;
+	long width;
+	long height;
+} SPIRect;
+
 /**
  *AccessibleComponentLayer:
  *@SPI_LAYER_INVALID: The layer cannot be determined or is somehow undefined.
@@ -1015,12 +1022,15 @@ char *       AccessibleTableColumnDescriptionChangedEvent_getDescriptionString (
 char *       AccessibleDescriptionChangedEvent_getDescriptionString (const AccessibleEvent *e);
 
 char *       AccessibleNameChangedEvent_getNameString (const AccessibleEvent *e);
+SPIRect *    AccessibleBoundsChangedEvent_getNewBounds (const AccessibleEvent *e);
 
 /* Misc methods and error handling */
 void SPI_freeString (char *s);
 
 char* SPI_dupString (char *s);
 
+void SPI_freeRect (SPIRect *rect);
+
 SPIBoolean SPI_exceptionHandlerPush (SPIExceptionHandler *handler);
 
 SPIExceptionHandler* SPI_exceptionHandlerPop (void);
