@@ -118,7 +118,10 @@ spi_init_role_lookup_table (Accessibility_Role *role_table)
   role_table [ATK_ROLE_FOOTER] =              Accessibility_ROLE_FOOTER;
   role_table [ATK_ROLE_PARAGRAPH] =           Accessibility_ROLE_PARAGRAPH;
   role_table [ATK_ROLE_RULER] =               Accessibility_ROLE_RULER;
-  
+  role_table [ATK_ROLE_APPLICATION] =         Accessibility_ROLE_APPLICATION;
+  role_table [ATK_ROLE_AUTOCOMPLETE] =        Accessibility_ROLE_AUTOCOMPLETE;
+  role_table [ATK_ROLE_EDITBAR] =             Accessibility_ROLE_EDITBAR;
+  role_table [ATK_ROLE_EMBEDDED] =            Accessibility_ROLE_EMBEDDED;
   return TRUE;
 }
 
@@ -392,7 +395,7 @@ impl_accessibility_accessible_get_role_name (PortableServer_Servant servant,
   AtkRole    role;
   AtkObject *object = get_atkobject_from_servant (servant);
 
-  g_return_val_if_fail (object != NULL, 0);
+  g_return_val_if_fail (object != NULL, NULL);
 
   role = atk_object_get_role (object);
 
@@ -403,6 +406,28 @@ impl_accessibility_accessible_get_role_name (PortableServer_Servant servant,
     return CORBA_string_dup ("");
 }
 
+/*
+ * CORBA Accessibility::Accessible::getLocalizedRole method implementation
+ */
+static CORBA_char *
+impl_accessibility_accessible_get_local_role_name (PortableServer_Servant servant,
+						   CORBA_Environment     *ev)
+{
+  const gchar     *role_name;
+  AtkRole    role;
+  AtkObject *object = get_atkobject_from_servant (servant);
+
+  g_return_val_if_fail (object != NULL, NULL);
+
+  role = atk_object_get_role (object);
+
+  role_name = atk_role_get_localized_name (role);
+  if (role_name)
+    return CORBA_string_dup (role_name);
+  else
+    return CORBA_string_dup ("");
+}
+
 static void
 spi_accessible_class_init (SpiAccessibleClass *klass)
 {
@@ -422,6 +447,7 @@ spi_accessible_class_init (SpiAccessibleClass *klass)
         epv->getState = impl_accessibility_accessible_get_state;
         epv->getRole = impl_accessibility_accessible_get_role;
         epv->getRoleName = impl_accessibility_accessible_get_role_name;
+	epv->getLocalizedRoleName = impl_accessibility_accessible_get_local_role_name;
 }
 
 static void
