@@ -2,7 +2,8 @@
  * AT-SPI - Assistive Technology Service Provider Interface
  * (Gnome Accessibility Project; http://developer.gnome.org/projects/gap)
  *
- * Copyright 2001 Sun Microsystems Inc.
+ * Copyright 2001, 2002 Sun Microsystems Inc.,
+ * Copyright 2001, 2002 Ximian, Inc.
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -22,6 +23,7 @@
 
 /* value.c : implements the Value interface */
 #include <config.h>
+#include <math.h>
 #include <stdio.h>
 #include <libspi/value.h>
 
@@ -55,7 +57,7 @@ impl__set_currentValue (PortableServer_Servant _servant,
 BONOBO_TYPE_FUNC_FULL (SpiValue,
 		       Accessibility_Value,
 		       PARENT_TYPE,
-		       spi_value);
+		       spi_value)
 
 
 static void
