@@ -39,6 +39,7 @@
 #include <X11/extensions/XTest.h>
 #include <X11/XKBlib.h>
 #define XK_MISCELLANY
+#define XK_LATIN1
 #include <X11/keysymdef.h>
 
 #ifdef HAVE_XEVIE
@@ -55,6 +56,16 @@
 #include "../libspi/spi-private.h"
 #include "deviceeventcontroller.h"
 
+KeySym ucs2keysym (long ucs);
+long keysym2ucs(KeySym keysym); 
+
+#define CHECK_RELEASE_DELAY 20
+#define BIT(c, x)       (c[x/8]&(1<<(x%8)))
+static guint check_release_handler = 0;
+static Accessibility_DeviceEvent pressed_event;
+static SpiDEController *saved_controller; 
+static void wait_for_release_event (XEvent *event, SpiDEController *controller);
+
 /* Our parent Gtk object type */
 #define PARENT_TYPE BONOBO_TYPE_OBJECT
 
@@ -115,6 +126,9 @@ typedef struct {
   unsigned int xkb_latch_mask;
   unsigned int pending_xkb_mod_relatch_mask;
   XkbDescPtr xkb_desc;
+  KeyCode reserved_keycode;
+  KeySym reserved_keysym;
+  guint  reserved_reset_timeout;
 } DEControllerPrivateData;
 
 static void     spi_controller_register_with_devices          (SpiDEController           *controller);
@@ -192,11 +206,77 @@ keysym_mod_mask (KeySym keysym, KeyCode keycode)
 	return retval;
 }
 
+static gboolean
+spi_dec_replace_map_keysym (DEControllerPrivateData *priv, KeyCode keycode, KeySym keysym)
+{
+#ifdef HAVE_XKB
+  Display *dpy = spi_get_display ();
+  XkbDescPtr desc;
+  if (!(desc = XkbGetMap (dpy, XkbAllMapComponentsMask, XkbUseCoreKbd)))
+    {
+      fprintf (stderr, "ERROR getting map\n");
+    }
+  XFlush (dpy);
+  XSync (dpy, False);
+  if (desc && desc->map)
+    {
+      gint offset = desc->map->key_sym_map[keycode].offset;
+      long old_sym = desc->map->syms[offset]; 
+      desc->map->syms[offset] = keysym; 
+    }
+  else
+    {
+      fprintf (stderr, "Error changing key map: empty server structure\n");
+    }		
+  XkbSetMap (dpy, XkbAllMapComponentsMask, desc);
+  /**
+   *  FIXME: the use of XkbChangeMap, and the reuse of the priv->xkb_desc structure, 
+   * would be far preferable.
+   * HOWEVER it does not seem to work using XFree 4.3. 
+   **/
+  /*	    XkbChangeMap (dpy, priv->xkb_desc, priv->changes); */
+  XFlush (dpy);
+  XSync (dpy, False);
+  XkbFreeKeyboard (desc, 0, TRUE);
+
+  return TRUE;
+#else
+  return FALSE;
+#endif
+}
+
+static gboolean
+spi_dec_reset_reserved (gpointer data)
+{
+  DEControllerPrivateData *priv = data;
+  spi_dec_replace_map_keysym (priv, priv->reserved_keycode, priv->reserved_keysym);
+  priv->reserved_reset_timeout = 0;
+  return FALSE;
+}
+
 static KeyCode
-keycode_for_keysym (long keysym, unsigned int *modmask)
+keycode_for_keysym (SpiDEController *controller, long keysym, unsigned int *modmask)
 {
 	KeyCode keycode = 0;
 	keycode = XKeysymToKeycode (spi_get_display (), (KeySym) keysym);
+	if (!keycode) 
+	{
+		DEControllerPrivateData *priv = (DEControllerPrivateData *)
+			g_object_get_qdata (G_OBJECT (controller), spi_dec_private_quark);
+		/* if there's no keycode available, fix it */
+		if (spi_dec_replace_map_keysym (priv, priv->reserved_keycode, keysym))
+		{
+			keycode = priv->reserved_keycode;
+			/* 
+			 * queue a timer to restore the old keycode.  Ugly, but required 
+			 * due to races / asynchronous X delivery.   
+			 * Long-term fix is to extend the X keymap here instead of replace entries.
+			 */
+			priv->reserved_reset_timeout = g_timeout_add (500, spi_dec_reset_reserved, priv);
+		}		
+		*modmask = 0;
+		return keycode;
+	}
 	if (modmask) 
 		*modmask = keysym_mod_mask (keysym, keycode);
 	return keycode;
@@ -540,6 +620,7 @@ spi_dec_init_mouse_listener (SpiRegistry *registry)
   Display *display = spi_get_display ();
   g_timeout_add (100, spi_dec_poll_mouse_idle, registry);
 
+#ifdef GRAB_BUTTON
   if (display)
     {
       if (XGrabButton (display, AnyButton, AnyModifier,
@@ -556,24 +637,31 @@ spi_dec_init_mouse_listener (SpiRegistry *registry)
       fprintf (stderr, "mouse buttons grabbed\n");
 #endif
     }
+#endif
 }
 
 /**
  * Eventually we can use this to make the marshalling of mask types
  * more sane, but for now we just use this to detect 
- * the use of 'virtual' masks such as Mumlock and convert them to
+ * the use of 'virtual' masks such as numlock and convert them to
  * system-specific mask values (i.e. ModMask).
  * 
  **/
 static Accessibility_ControllerEventMask
 spi_dec_translate_mask (Accessibility_ControllerEventMask mask)
 {
-  DEControllerPrivateData *priv;
+  Accessibility_ControllerEventMask tmp_mask;
+  gboolean has_numlock;
 
-  if (mask == SPI_KEYMASK_NUMLOCK) {
-    mask = _numlock_physical_mask;
-  }
-  return mask;
+  has_numlock = (mask & SPI_KEYMASK_NUMLOCK);
+  tmp_mask = mask;
+  if (has_numlock)
+    {
+      tmp_mask = mask ^ SPI_KEYMASK_NUMLOCK;
+      tmp_mask |= _numlock_physical_mask;
+    }
+ 
+  return tmp_mask;
 }
 
 static DEControllerKeyListener *
@@ -1007,13 +1095,39 @@ global_filter_fn (GdkXEvent *gdk_xevent, GdkEvent *event, gpointer data)
     {
       if (controller->xevie_display == NULL)
         {
-          gboolean is_consumed =
+          gboolean is_consumed;
+
+          is_consumed =
             spi_device_event_controller_forward_key_event (controller, xevent);
 
           if (is_consumed)
-            XAllowEvents (spi_get_display (), AsyncKeyboard, CurrentTime);
+            {
+              int n_events;
+              int i;
+              XEvent next_event;
+              n_events = XPending (display);
+
+#ifdef SPI_KEYEVENT_DEBUG
+              g_print ("Number of events pending: %d\n", n_events);
+#endif
+              for (i = 0; i < n_events; i++)
+                {
+                  XNextEvent (display, &next_event);
+		  if (next_event.type != KeyPress &&
+		      next_event.type != KeyRelease)
+			g_warning ("Unexpected event type %d in queue", next_event.type);
+                 }
+
+              XAllowEvents (display, AsyncKeyboard, CurrentTime);
+              if (n_events)
+                XUngrabKeyboard (display, CurrentTime);
+            }
           else
-            XAllowEvents (spi_get_display (), ReplayKeyboard, CurrentTime);
+            {
+              if (xevent->type == KeyPress)
+                wait_for_release_event (xevent, controller);
+              XAllowEvents (display, ReplayKeyboard, CurrentTime);
+            }
         }
 
       return GDK_FILTER_CONTINUE;
@@ -1026,7 +1140,7 @@ global_filter_fn (GdkXEvent *gdk_xevent, GdkEvent *event, gpointer data)
     {
       XkbAnyEvent * xkb_ev = (XkbAnyEvent *) xevent;
       /* ugly but probably necessary...*/
-      XSynchronize (spi_get_display (), TRUE);
+      XSynchronize (display, TRUE);
 
       if (xkb_ev->xkb_type == XkbStateNotify)
         {
@@ -1088,7 +1202,7 @@ global_filter_fn (GdkXEvent *gdk_xevent, GdkEvent *event, gpointer data)
 	}
         else
 	       DBG (2, g_warning ("XKB event %d\n", xkb_ev->xkb_type));
-      XSynchronize (spi_get_display (), FALSE);
+      XSynchronize (display, FALSE);
     }
   
   return GDK_FILTER_CONTINUE;
@@ -1127,12 +1241,56 @@ spi_controller_register_with_devices (SpiDEController *controller)
 				      &priv->xkb_base_error_code, NULL, NULL);
   if (priv->have_xkb)
     {
-      priv->xkb_desc = XkbGetMap (spi_get_display (), 0, XkbUseCoreKbd);
+      gint i;
+      guint64 reserved = 0;
+      priv->xkb_desc = XkbGetMap (spi_get_display (), XkbKeySymsMask, XkbUseCoreKbd);
       XkbSelectEvents (spi_get_display (),
 		       XkbUseCoreKbd,
 		       XkbStateNotifyMask, XkbStateNotifyMask);	    
       _numlock_physical_mask = XkbKeysymToModifiers (spi_get_display (), 
 						     XK_Num_Lock);
+      for (i = priv->xkb_desc->max_key_code; i >= priv->xkb_desc->min_key_code; --i)
+      {
+	  if (priv->xkb_desc->map->key_sym_map[i].kt_index[0] == XkbOneLevelIndex)
+	  { 
+	      if (XKeycodeToKeysym (spi_get_display (), i, 0) != 0)
+	      {
+		  /* don't use this one if there's a grab client! */
+		  gdk_error_trap_push ();
+		  XGrabKey (spi_get_display (), i, 0, 
+			    gdk_x11_get_default_root_xwindow (),
+			    TRUE,
+			    GrabModeSync, GrabModeSync);
+		  XSync (spi_get_display (), TRUE);
+		  XUngrabKey (spi_get_display (), i, 0, 
+			      gdk_x11_get_default_root_xwindow ());
+		  if (!gdk_error_trap_pop ())
+		  {
+		      reserved = i;
+		      break;
+		  }
+	      }
+	  }
+      }
+      if (reserved) 
+      {
+	  priv->reserved_keycode = reserved;
+	  priv->reserved_keysym = XKeycodeToKeysym (spi_get_display (), reserved, 0);
+      }
+      else
+      { 
+	  priv->reserved_keycode = XKeysymToKeycode (spi_get_display (), XK_numbersign);
+	  priv->reserved_keysym = XK_numbersign;
+      }
+#ifdef SPI_RESERVED_DEBUG
+      unsigned sym = 0;
+      sym = XKeycodeToKeysym (spi_get_display (), reserved, 0);
+      fprintf (stderr, "%x\n", sym);
+      fprintf (stderr, "setting the reserved keycode to %d (%s)\n", 
+	       reserved, 
+	       XKeysymToString (XKeycodeToKeysym (spi_get_display (),
+                                                            reserved, 0)));
+#endif
     }	
 
   gdk_window_add_filter (NULL, global_filter_fn, controller);
@@ -1322,7 +1480,7 @@ spi_controller_notify_keylisteners (SpiDEController                 *controller,
 }
 
 static gboolean
-spi_clear_error_state ()
+spi_clear_error_state (void)
 {
 	gboolean retval = spi_error_code != 0;
 	spi_error_code = 0;
@@ -1335,7 +1493,7 @@ spi_keystroke_from_x_key_event (XKeyEvent *x_key_event)
   Accessibility_DeviceEvent key_event;
   KeySym keysym;
   const int cbuf_bytes = 20;
-  char cbuf [cbuf_bytes+1];
+  char cbuf [21];
   int nbytes;
 
   nbytes = XLookupString (x_key_event, cbuf, cbuf_bytes, &keysym, NULL);  
@@ -1434,8 +1592,8 @@ spi_keystroke_from_x_key_event (XKeyEvent *x_key_event)
 	    gunichar c;
 	    cbuf[nbytes] = '\0'; /* OK since length is cbuf_bytes+1 */
             key_event.event_string = CORBA_string_dup (cbuf);
-	    c = g_utf8_get_char_validated (cbuf, nbytes);
-	    if ((c > 0) && g_unichar_isprint (c))
+	    c = keysym2ucs (keysym);
+	    if (c > 0 && !g_unichar_iscntrl (c))
 	      {
 	        key_event.is_text = CORBA_TRUE; 
 		/* incorrect for some composed chars? */
@@ -1475,6 +1633,7 @@ spi_keystroke_from_x_key_event (XKeyEvent *x_key_event)
      (x_key_event->state & Mod1Mask)?"Alt-":"",
      ((x_key_event->state & ShiftMask)^(x_key_event->state & LockMask))?
      g_ascii_toupper (keysym) : g_ascii_tolower (keysym));
+  fprintf (stderr, "serial: %x Time: %x\n", x_key_event->serial, x_key_event->time);
 #endif /* SPI_DEBUG */
   return key_event;	
 }
@@ -1485,6 +1644,7 @@ spi_controller_update_key_grabs (SpiDEController           *controller,
 {
   GList *l, *next;
   gboolean   update_failed = FALSE;
+  KeyCode keycode;
   
   g_return_val_if_fail (controller != NULL, FALSE);
 
@@ -1497,6 +1657,8 @@ spi_controller_update_key_grabs (SpiDEController           *controller,
    *
    * ControlMask grabs are broken, must be in use already
    */
+  if (recv)
+    keycode = keycode_for_keysym (controller, recv->id, NULL);
   for (l = controller->keygrabs_list; l; l = next)
     {
       gboolean do_remove;
@@ -1507,7 +1669,7 @@ spi_controller_update_key_grabs (SpiDEController           *controller,
 
       re_issue_grab = recv &&
 	      (recv->modifiers & grab_mask->mod_mask) &&
-	      (grab_mask->key_val == keycode_for_keysym (recv->id, NULL));
+	      (grab_mask->key_val == keycode);
 
 #ifdef SPI_DEBUG
       fprintf (stderr, "mask=%lx %lx (%c%c) %s\n",
@@ -1877,6 +2039,7 @@ dec_synth_keycode_press (SpiDEController *controller,
 	}
         XTestFakeKeyEvent (spi_get_display (), keycode, True, time);
 	priv->last_press_keycode = keycode;
+	XFlush (spi_get_display ());
 	XSync (spi_get_display (), False);
 	gettimeofday (&priv->last_press_time, NULL);
 	return TRUE;
@@ -1948,45 +2111,7 @@ dec_unlock_modifiers (SpiDEController *controller, unsigned modifiers)
 static KeySym
 dec_keysym_for_unichar (SpiDEController *controller, gunichar unichar)
 {
-	/* TODO: table lookups within a range, for various code pages! */
-	KeySym keysym = NoSymbol;
-
-	if (unichar >= 0x20 && unichar < 0x7f) { /* Basic Latin/ASCII */
-		keysym = (KeySym) unichar;
-	}
-	else if (unichar >= 0xa0 && unichar <= 0xff) { /* Latin 1 extensions */
-		keysym = (KeySym) unichar;
-	}
-	else if (unichar >= 0x100 && unichar <= 0x233) { /* unfortunately the mapping gets nasty for Latin-2 and 3... help! */
-		keysym = NoSymbol;
-	}
-	else if (unichar >= 0x7c1 && unichar <= 0x3a1) { /* let's try Greek anyway... */
-		keysym = (KeySym) (0x391 + (unichar - 0x7c1));
-	}
-	else if (unichar >= 0x3a3 && unichar <= 0x3a9) { /* let's try Greek anyway... */
-		keysym = (KeySym) (0x7d2 + (unichar - 0x3a3));
-	}
-	else if (unichar >= 0x3b1 && unichar <= 0x3c1) { /* let's try Greek anyway... */
-		keysym = (KeySym) (0x7e1 + (unichar - 0x3b1));
-	}
-	else if (unichar == 0x3c2) {
-		keysym = (KeySym) 0x7f3; /* XK_Greek_finalsmallsigma; */
-	}
-	else if (unichar >= 0x3c3 && unichar <= 0x3c9) { /* let's try Greek anyway... */
-		keysym = (KeySym) (0x7f2 + (unichar - 0x3c2));
-	}	
-	else if (unichar >= 0x5d0 && unichar <= 0x5ea) { /* Hebrew basics */
-		/* probably broken :-) */
-		keysym = (KeySym) (0xce0 + (unichar - 0x5d0));
-	}	
-	else if (unichar >= 0x30a1 && unichar <= 0x30ab) { /* partial katakana support */
-		/* TODO: complete! */
-		keysym = (KeySym) (0x4b1 + (unichar - 0x30a1)/2);
-	}
-	else if (unichar >= 0x20a0 && unichar <= 0x20ac) { /* currency */
-		keysym = (KeySym) unichar; /* how convenient ;-) */
-	}
-	return keysym;
+	return ucs2keysym ((long) unichar);
 }
 
 static gboolean
@@ -1995,7 +2120,7 @@ dec_synth_keysym (SpiDEController *controller, KeySym keysym)
 	KeyCode key_synth_code;
 	unsigned int modifiers, synth_mods, lock_mods;
 
-	key_synth_code = keycode_for_keysym (keysym, &synth_mods);
+	key_synth_code = keycode_for_keysym (controller, keysym, &synth_mods);
 
 	if ((key_synth_code == 0) || (synth_mods == 0xFF)) return FALSE;
 
@@ -2003,12 +2128,14 @@ dec_synth_keysym (SpiDEController *controller, KeySym keysym)
 	modifiers = dec_get_modifier_state (controller);
 	/* side-effect; we may unset mousebutton modifiers here! */
 
+	lock_mods = 0;
 	if (synth_mods != modifiers) {
 		lock_mods = synth_mods & ~modifiers;
 		dec_lock_modifiers (controller, lock_mods);
 	}
 	dec_synth_keycode_press (controller, key_synth_code);
 	dec_synth_keycode_release (controller, key_synth_code);
+
 	if (synth_mods != modifiers) 
 		dec_unlock_modifiers (controller, lock_mods);
 	return TRUE;
@@ -2060,6 +2187,7 @@ dec_synth_keystring (SpiDEController *controller, const CORBA_char *keystring)
 			keystring = g_utf8_next_char (keystring); 
 		}
 		keysyms[i++] = 0;
+		XSynchronize (spi_get_display (), TRUE);
 		for (i = 0; keysyms[i]; ++i) {
 			if (!dec_synth_keysym (controller, keysyms[i])) {
 #ifdef SPI_DEBUG
@@ -2070,6 +2198,7 @@ dec_synth_keystring (SpiDEController *controller, const CORBA_char *keystring)
 				break;
 			}
 		}
+		XSynchronize (spi_get_display (), FALSE);
 	}
 	g_free (keysyms);
 
@@ -2091,6 +2220,7 @@ impl_generate_keyboard_event (PortableServer_Servant           servant,
   SpiDEController *controller =
 	SPI_DEVICE_EVENT_CONTROLLER (bonobo_object (servant));
   long key_synth_code;
+  gint err;
   KeySym keysym;
 
 #ifdef SPI_DEBUG
@@ -2134,9 +2264,9 @@ impl_generate_keyboard_event (PortableServer_Servant           servant,
 			       keystring);
 	      break;
     }
-  if (gdk_error_trap_pop ())
+  if ((err = gdk_error_trap_pop ()))
     {
-      DBG (-1, g_warning ("Error emitting keystroke"));
+      DBG (-1, g_warning ("Error [%d] emitting keystroke", err));
     }
   if (synth_type == Accessibility_KEY_SYM) {
     keysym = keycode;
@@ -2382,6 +2512,46 @@ spi_device_event_controller_new (SpiRegistry *registry)
   return retval;
 }
 
+static gboolean
+is_key_released (KeyCode code)
+{
+  char keys[32];
+  int down;
+  int i;
+
+  XQueryKeymap (spi_get_display (), keys);
+  down = BIT (keys, code);
+  return (down == 0);
+}
+
+static gboolean
+check_release (gpointer data)
+{
+  gboolean released;
+  Accessibility_DeviceEvent *event = (Accessibility_DeviceEvent *)data;
+  KeyCode code = event->hw_code;
+  CORBA_Environment ev;
+
+  released = is_key_released (code);
+
+  if (released)
+    {
+      check_release_handler = 0;
+      event->type = Accessibility_KEY_RELEASED_EVENT;
+      ev._major = CORBA_NO_EXCEPTION;
+      spi_controller_notify_keylisteners (saved_controller, event, CORBA_TRUE, &ev);
+    }
+  return (released == 0);
+}
+
+static void wait_for_release_event (XEvent          *event,
+                                    SpiDEController *controller)
+{
+  pressed_event = spi_keystroke_from_x_key_event ((XKeyEvent *) event);
+  saved_controller = controller;
+  check_release_handler = g_timeout_add (CHECK_RELEASE_DELAY, check_release, &pressed_event);
+}
+
 BONOBO_TYPE_FUNC_FULL (SpiDEController,
 		       Accessibility_DeviceEventController,
 		       PARENT_TYPE,
