@@ -22,7 +22,7 @@
  */
 
 #include <stdio.h>
-#include <strings.h>
+#include <string.h>
 #include <stdlib.h>
 #include "../cspi/spi-private.h"
 
@@ -331,10 +331,10 @@ text_chunk_list_head_clip (GList *text_chunk_list,
 {
 	GList *target, *iter = next, *prev;
 	prev = iter->prev;
-//	if (chunk->string && strlen (chunk->string)) { 
+/*	if (chunk->string && strlen (chunk->string)) { */
 		text_chunk_list =
 			g_list_insert_before (text_chunk_list, next, chunk);
-//	}
+/*	}*/
 	while (iter && CHUNK_BOUNDS_SPANS_END (chunk, (TextChunk *)iter->data)) {
 #ifdef CLIP_DEBUG			
 			fprintf (stderr, "deleting %s\n",
@@ -1128,7 +1128,7 @@ report_screen_review_line (const AccessibleEvent *event, void *user_data)
 }
 
 void
-test_exit ()
+test_exit (void)
 {
   SPI_deregisterGlobalEventListenerAll (mouseclick_listener);
   AccessibleEventListener_unref (mouseclick_listener);
