@@ -43,7 +43,7 @@ static SPIBoolean report_command_key_event  (const AccessibleKeystroke *stroke,
 static SPIBoolean report_ordinary_key_event (const AccessibleKeystroke *stroke, void *user_data);
 static void get_environment_vars (void);
 
-static int _festival_init ();
+static int _festival_init (void);
 static void _festival_say (const char *text, const char *voice, SPIBoolean shutup);
 static void _festival_write (const char *buff, int fd);
 
@@ -114,6 +114,7 @@ main (int argc, char **argv)
   SPI_registerGlobalEventListener (text_listener, "object:text-changed"); 
   SPI_registerGlobalEventListener (button_listener, "Gtk:GtkWidget:button-press-event");
   SPI_registerGlobalEventListener (window_listener, "window:minimize");
+  SPI_registerGlobalEventListener (window_listener, "window:activate");
   n_desktops = SPI_getDesktopCount ();
 
   for (i=0; i<n_desktops; ++i)
@@ -127,7 +128,7 @@ main (int argc, char **argv)
         {
           application = Accessible_getChildAtIndex (desktop, j);
 	  s = Accessible_getName (application);
-          fprintf (stderr, "app %d name: %s\n", j, s);
+          fprintf (stderr, "app %d name: %s\n", j, s ? s : "(nil)");
 #ifdef PRINT_TREE
 	  print_accessible_tree (application, "*");
 #endif
@@ -222,7 +223,9 @@ print_accessible_tree (Accessible *accessible, char *prefix)
 	name = Accessible_getName (accessible);
 	role_name = Accessible_getRoleName (accessible);
 	fprintf (stdout, "%sAccessible [%s] \"%s\"; parent [%s] %s.\n",
-		 prefix, role_name, name, parent_role, parent_name);
+		 prefix, role_name, name ? name : "(nil)",
+		 parent_role ? parent_role : "(nil)",
+		 parent_name ? parent_name : "(nil)");
 	SPI_freeString (name);
 	SPI_freeString (role_name);
 	SPI_freeString (parent_name);
@@ -350,6 +353,10 @@ void
 report_window_event (const AccessibleEvent *event, void *user_data)
 {
   fprintf (stderr, "%s event received\n", event->type);
+  if (!strcmp (event->type, "window:activate"))
+  {
+      print_accessible_tree (event->source, "window");
+  }
 }
 
 void
@@ -445,7 +452,7 @@ check_property_change (const AccessibleEvent *event, void *user_data)
 }
 
 static void
-simple_at_exit ()
+simple_at_exit (void)
 {
   SPI_deregisterGlobalEventListenerAll (focus_listener);
   AccessibleEventListener_unref        (focus_listener);
@@ -525,7 +532,7 @@ report_ordinary_key_event (const AccessibleKeystroke *key, void *user_data)
 }
 
 static int
-_festival_init ()
+_festival_init (void)
 {
   int fd;
   struct sockaddr_in name;
